# -*- coding: utf-8 -*-
# Copyright (c) 2016, 2017, 2018, 2019, 2020, Sqreen. All rights reserved.
# Please refer to our terms for more information:
#
#     https://www.sqreen.io/terms.html
#

__author__ = "Sqreen"
__copyright__ = "Copyright 2016, 2017, 2018, 2019, 2020, Sqreen"
__email__ = "contact@sqreen.io"
__license__ = "proprietary"
# The lib version is always contained at the beginning of __version__
__version__ = "0.5.1.0.1"
__lib_version__ = "0.5.1"
